package com.atguigu.ssyx.product.service.impl;

import com.atguigu.ssyx.model.product.SkuAttrValue;
import com.atguigu.ssyx.model.product.SkuImage;
import com.atguigu.ssyx.model.product.SkuInfo;
import com.atguigu.ssyx.model.product.SkuPoster;
import com.atguigu.ssyx.mq.constant.MqConst;
import com.atguigu.ssyx.mq.service.RabbitService;
import com.atguigu.ssyx.product.mapper.SkuInfoMapper;
import com.atguigu.ssyx.product.service.SkuAttrValueService;
import com.atguigu.ssyx.product.service.SkuImageService;
import com.atguigu.ssyx.product.service.SkuInfoService;
import com.atguigu.ssyx.product.service.SkuPosterService;
import com.atguigu.ssyx.vo.product.SkuInfoQueryVo;
import com.atguigu.ssyx.vo.product.SkuInfoVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.jsonwebtoken.lang.Collections;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * <p>
 * sku信息 服务实现类
 * </p>
 *
 * @author bunny
 * @since 2024-04-03
 */
@Service
public class SkuInfoServiceImpl extends ServiceImpl<SkuInfoMapper, SkuInfo> implements SkuInfoService {
    @Autowired
    private SkuAttrValueService skuAttrValueService;
    @Autowired
    private SkuImageService skuImageService;
    @Autowired
    private SkuPosterService skuPosterService;
    @Autowired
    private RabbitService rabbitService;

    /**
     * 获取sku分页列表
     *
     * @param pageParam 分页查询条件
     * @param vo        商品查询条件
     * @return 分页结果
     */
    @Override
    public IPage<SkuInfo> selectPage(Page<SkuInfo> pageParam, SkuInfoQueryVo vo) {
        String keyword = vo.getKeyword();
        String skuType = vo.getSkuType();
        Long categoryId = vo.getCategoryId();

        // 封装查询条件
        LambdaQueryWrapper<SkuInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.like(!StringUtils.isEmpty(keyword), SkuInfo::getSkuName, keyword);
        wrapper.eq(!StringUtils.isEmpty(skuType), SkuInfo::getSkuType, skuType);
        wrapper.eq(categoryId != null, SkuInfo::getCategoryId, categoryId);

        // 返回结果
        return page(pageParam, wrapper);
    }

    /**
     * 商品添加方法
     *
     * @param skuInfoVo 商品信息
     */
    @Override
    public void saveSkuInfo(SkuInfoVo skuInfoVo) {
        // 添加sku基本信息
        SkuInfo skuInfo = new SkuInfo();
        BeanUtils.copyProperties(skuInfoVo, skuInfo);
        baseMapper.insert(skuInfo);
        // 保存sku海报
        List<SkuPoster> skuPosterList = skuInfoVo.getSkuPosterList();
        if (!Collections.isEmpty(skuPosterList)) {
            skuPosterList.forEach(skuPoster -> skuPoster.setSkuId(skuInfo.getId()));
            skuPosterService.saveBatch(skuPosterList);
        }
        // 保存sku图片
        List<SkuImage> skuImagesList = skuInfoVo.getSkuImagesList();
        if (!Collections.isEmpty(skuImagesList)) {
            skuImagesList.forEach(skuImage -> skuImage.setSkuId(skuImage.getId()));
            skuImageService.saveBatch(skuImagesList);
        }
        // 保存sku属性
        List<SkuAttrValue> skuAttrValueList = skuInfoVo.getSkuAttrValueList();
        if (!Collections.isEmpty(skuImagesList)) {
            skuAttrValueList.forEach(skuAttrValue -> skuAttrValue.setSkuId(skuInfo.getId()));
            skuAttrValueService.saveBatch(skuAttrValueList);
        }
    }

    /**
     * 获取商品SKU信息
     *
     * @param id 商品id
     * @return 商品信息
     */
    @Override
    public SkuInfoVo getSkuInfoVo(Long id) {
        SkuInfoVo skuInfoVo = new SkuInfoVo();

        // 根据id查询sku基本信息
        SkuInfo skuInfo = baseMapper.selectById(id);

        // 根据id查询商品图片列表
        List<SkuImage> skuImageList = skuImageService.getImageListBySkuId(id);

        // 根据id查询商品海报列表
        List<SkuPoster> skuPosterList = skuPosterService.getPosterListBySkuId(id);

        // 根据id查询商品属性信息列表
        List<SkuAttrValue> skuAttrValueList = skuAttrValueService.getAttrValueListBySkuId(id);

        BeanUtils.copyProperties(skuInfo, skuInfoVo);
        skuInfoVo.setSkuAttrValueList(skuAttrValueList);
        skuInfoVo.setSkuImagesList(skuImageList);
        skuInfoVo.setSkuPosterList(skuPosterList);

        // 封装数据返回
        return skuInfoVo;
    }

    /**
     * 修改商品sku信息
     *
     * @param skuInfoVo 商品信息
     */
    @Override
    public void updateSkuInfo(SkuInfoVo skuInfoVo) {
        // 修改sku基本信息
        SkuInfo skuInfo = new SkuInfo();
        BeanUtils.copyProperties(skuInfo, skuInfo);
        baseMapper.updateById(skuInfo);

        // 海报信息
        Long skuId = skuInfoVo.getId();
        // 创建查询条件，并删除之前的海报内容
        LambdaQueryWrapper<SkuPoster> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SkuPoster::getSkuId, skuId);
        skuPosterService.remove(wrapper);

        // 设置海报图片skuId值，并完成添加
        List<SkuPoster> skuPosterList = skuInfoVo.getSkuPosterList();
        if (!CollectionUtils.isEmpty(skuPosterList)) {
            skuPosterList.forEach(skuPoster -> skuPoster.setSkuId(skuId));
            skuPosterService.saveBatch(skuPosterList);
        }

        // 商品图片
        skuImageService.remove(Wrappers.<SkuImage>lambdaQuery().eq(SkuImage::getSkuId, skuId));
        List<SkuImage> skuImagesList = skuInfoVo.getSkuImagesList();
        if (!Collections.isEmpty(skuImagesList)) {
            skuImagesList.forEach(skuImage -> skuImage.setSkuId(skuId));
            skuImageService.saveBatch(skuImagesList);
        }

        // 商品属性
        skuAttrValueService.remove(Wrappers.<SkuAttrValue>lambdaQuery().eq(SkuAttrValue::getSkuId, skuId));
        List<SkuAttrValue> skuAttrValueList = skuInfoVo.getSkuAttrValueList();
        if (!Collections.isEmpty(skuAttrValueList)) {
            skuAttrValueList.forEach(skuAttrValue -> skuAttrValue.setSkuId(skuId));
            skuAttrValueService.saveBatch(skuAttrValueList);
        }
    }

    /**
     * 商品审核
     *
     * @param skuId  商品ID
     * @param status 商品状态
     */
    @Override
    public void check(Long skuId, Integer status) {
        SkuInfo skuInfo = new SkuInfo();
        skuInfo.setId(skuId);
        skuInfo.setCheckStatus(status);
        updateById(skuInfo);
    }

    /**
     * 商品上架
     *
     * @param skuId  商品ID
     * @param status 商品状态
     */
    @Override
    public void publish(Long skuId, Integer status) {
        SkuInfo skuInfo = new SkuInfo();
        skuInfo.setId(skuId);
        if (status == 1) {
            skuInfo.setPublishStatus(status);
            // 商品上架，发送MQ消息
            rabbitService.sendMessage(MqConst.EXCHANGE_GOODS_DIRECT, MqConst.ROUTING_GOODS_UPPER, skuId);
        } else {
            skuInfo.setPublishStatus(0);
            // 商品下架：发送mq消息同步es
            rabbitService.sendMessage(MqConst.EXCHANGE_GOODS_DIRECT, MqConst.ROUTING_GOODS_LOWER, skuId);
        }
        baseMapper.updateById(skuInfo);
    }

    /**
     * 新人专享
     *
     * @param skuId  商品ID
     * @param status 商品状态
     */
    @Override
    public void isNewPerson(Long skuId, Integer status) {
        SkuInfo skuInfo = new SkuInfo();
        skuInfo.setId(skuId);
        skuInfo.setIsNewPerson(status);
        baseMapper.updateById(skuInfo);
    }
}
